/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include <libgen.h>
#include <glib.h>
#include <config.h>
#include "bibletime.h"
#include "log.h"
#include "bible.h"


#define STAGE_ZERO 0
#define STAGE_CHECK_DCOP 1
#define STAGE_COMMUNICATE 2
#define STAGE_WAIT_RETRY 1000
#define STAGE_RETRY 1600


#define BIBLETIME "bibletime"


BibleTime::BibleTime (bool dummy)
/*
This controls the BibleTime program.
It uses KDE's DCOP protocol.
*/
{
  connected = false;
  clear ();
}


BibleTime::~BibleTime ()
{
  disconnect (false);
}


void BibleTime::clear ()
{
  run_stages = false;
  disconnect ();
  stage = STAGE_ZERO;
  stages_running = false;
  connected = false;
}


void BibleTime::connect ()
{
  clear ();
  run_stages = true;
}


void BibleTime::tick ()
{
  if (run_stages)
    runstages ();
}


void
BibleTime::disconnect (bool message)
{
  if (connected) {
    // Only write a message when "message" is true.
    // This is done to prevent writing messages when 
    // Bibledit shuts down. This message would show up the next time
    // Bibledit starts. This would be confusing.
    if (message)
      log ("Disconnected");
  }
  connected = false;
}


void
BibleTime::runstages ()
{
  if (stages_running)
    return;
  stages_running = true;
  switch (stage)
    {
    case STAGE_ZERO:
      {
        // Check if BibleTime is running now.
        if (!program_is_running (BIBLETIME)) {
          log ("Program is not running");
          stage = STAGE_WAIT_RETRY;
          break;
        }
        stage++;
        break;
      }
    case STAGE_CHECK_DCOP:
      {
        // Check if dcop protocol is working, and get bibletime's name.
        bibledit_dcop_name.clear();
        FILE *stream;
        string command = "dcop";
        stream = popen (command.c_str (), "r");
        char buf[1024];
        while (fgets (buf, sizeof (buf), stream)) {
          command = buf;
          command = trim (command);
          if (command.find (BIBLETIME) != string::npos) {
            bibledit_dcop_name = command;
          }
        }
        pclose (stream);
        // Was bibledit's dcop name found?
        if (bibledit_dcop_name.empty()) {
          log ("No DCPOP connection possible");
          stage = STAGE_WAIT_RETRY;
          break;
        }
        // Ok, connection is possible.
        log ("Connected");
        connected = true;
        stage++;
        break;
      }
    case STAGE_COMMUNICATE:
      {
        break;
      }
    case STAGE_WAIT_RETRY:
      {
        log ("Will try to connect again after a minute");
        stage++;
        break;
      }
    case STAGE_RETRY:
      {
        stage = STAGE_ZERO;
        break;
      }
    default:
      {
        stage++;
      }
    }
  stages_running = false;
}


void
BibleTime::log (const string & message)
{
  log_message ("BibleTime: " + message);
}


void
BibleTime::sendreference (const string& book, const string& chapter, const string& verse)
{
  // Sends the reference to BibleTime.
  if (connected) {
    unsigned id = english_id_to_index (book);
    string osisbook = index_to_osis_id (id);
    string command;
    command = "dcop ";
    command.append (bibledit_dcop_name);
    command.append (" syncAllVerseBasedModules ");
    command.append (osisbook);
    command.append (".");
    command.append (chapter);
    command.append (".");
    command.append (verse);
    command.append (" > /dev/null");
    int result = system (command.c_str());
    if (result != 0) {
      connected = false;
      log ("Error sending data");
      reconnect ();
    }
  }
}


void BibleTime::reconnect ()
{
  disconnect (false);
  connect ();
}
