/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogfop.h"
#include "utilities.h"
#include "java.h"



FopDialog::FopDialog (Configuration* configuration)
{
  myconfiguration = configuration;

  fopdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (fopdialog), "FOP Setup");
  gtk_window_set_position (GTK_WINDOW (fopdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (fopdialog), TRUE);
  gtk_window_set_destroy_with_parent (GTK_WINDOW (fopdialog), TRUE);
  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (fopdialog), TRUE);
  gtk_window_set_skip_pager_hint (GTK_WINDOW (fopdialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (fopdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 5);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new ("Bibledit uses FOP (Formatting Objects Processor) to lay the text out on the page,\nand then print it.");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

  label2 = gtk_label_new ("To install FOP and use it, follow the steps below.\n");
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (vbox1), label2, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

  label3 = gtk_label_new ("1. Download a Java Runtime Environment suitable for your operating system.\nFor example the Java RE on http://www.java.com/en/download/linux_manual.jsp");
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (vbox1), label3, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS (label3, GTK_CAN_FOCUS);
  gtk_label_set_selectable (GTK_LABEL (label3), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);

  label4 = gtk_label_new ("2. Install it in a convenient directory on your system.");
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (vbox1), label4, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label4), 0, 0.5);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);

  label5 = gtk_label_new ("3. Enter the directory where you installed it:");
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (vbox2), label5, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, FALSE, 0);

  entry1 = gtk_entry_new ();
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (hbox1), entry1, TRUE, TRUE, 0);
  gtk_entry_set_activates_default (GTK_ENTRY (entry1), TRUE);

  button1 = gtk_button_new_from_stock ("gtk-open");
  gtk_widget_show (button1);
  gtk_box_pack_start (GTK_BOX (hbox1), button1, FALSE, FALSE, 0);

  label6 = gtk_label_new ("4. Download FOP from http://xml.apache.org/fop");
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (vbox1), label6, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS (label6, GTK_CAN_FOCUS);
  gtk_label_set_selectable (GTK_LABEL (label6), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label6), 0, 0.5);

  label7 = gtk_label_new ("5. Install it in a convenient directory on your system.");
  gtk_widget_show (label7);
  gtk_box_pack_start (GTK_BOX (vbox1), label7, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label7), 0, 0.5);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_box_pack_start (GTK_BOX (vbox1), vbox3, TRUE, TRUE, 0);

  label8 = gtk_label_new ("6. Enter the directory where you installed it:");
  gtk_widget_show (label8);
  gtk_box_pack_start (GTK_BOX (vbox3), label8, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label8), 0, 0.5);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox2, FALSE, FALSE, 0);

  entry2 = gtk_entry_new ();
  gtk_widget_show (entry2);
  gtk_box_pack_start (GTK_BOX (hbox2), entry2, TRUE, TRUE, 0);
  gtk_entry_set_activates_default (GTK_ENTRY (entry2), TRUE);

  button2 = gtk_button_new_from_stock ("gtk-open");
  gtk_widget_show (button2);
  gtk_box_pack_start (GTK_BOX (hbox2), button2, FALSE, FALSE, 0);

  dialog_action_area1 = GTK_DIALOG (fopdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (fopdialog), cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (fopdialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) okbutton, "clicked", G_CALLBACK (on_okbutton_clicked), gpointer(this));
  g_signal_connect ((gpointer) button1, "clicked", G_CALLBACK (on_button1_clicked), gpointer(this));
  g_signal_connect ((gpointer) button2, "clicked", G_CALLBACK (on_button2_clicked), gpointer(this));

  string java_home;
  java_home = java_home_environment_variable ();
  if (java_home.empty()) {
    gtk_entry_set_text (GTK_ENTRY (entry1), myconfiguration->java_home.c_str ());
  } else {
    gtk_entry_set_text (GTK_ENTRY (entry1), java_home.c_str ());
    gtk_widget_set_sensitive (entry1, false);
    gtk_label_set_text (GTK_LABEL (label5), "3. Done. Your system already knows about the directory.");
    gtk_widget_set_sensitive (button1, false);
  }
  gtk_entry_set_text (GTK_ENTRY (entry2), myconfiguration->fop_home.c_str ());

  gtk_widget_grab_focus (entry1);
  gtk_widget_grab_default (okbutton);
}


FopDialog::~FopDialog ()
{
  gtk_widget_destroy (fopdialog);
}


int FopDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (fopdialog));
}


void FopDialog::on_okbutton_clicked (GtkButton * button, gpointer user_data)
{
  ((FopDialog *) user_data)->on_okbutton ();
}


void FopDialog::on_okbutton ()
{
  myconfiguration->java_home = gtk_entry_get_text (GTK_ENTRY (entry1));
  myconfiguration->fop_home = gtk_entry_get_text (GTK_ENTRY (entry2));
}


void FopDialog::on_button1_clicked (GtkButton * button, gpointer user_data)
{
  ((FopDialog *) user_data)->on_button1 ();
}


void FopDialog::on_button1 ()
{
  GtkWidget *file_selector;
  /* Create the selector */
  file_selector = gtk_file_selection_new ("Please choose a directory");
  // Prevent the user from selecting a file.
  gtk_widget_hide (GTK_FILE_SELECTION (file_selector)->file_list);
  gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (file_selector));
  string directory = gtk_entry_get_text (GTK_ENTRY (entry1));
  directory.append ("/");
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_selector), directory.c_str());
  // Display that dialog
  gtk_widget_show (file_selector);
  int result = gtk_dialog_run (GTK_DIALOG (file_selector));
  directory = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
  gtk_widget_destroy (file_selector);
  // Handle the response:
  if (result == GTK_RESPONSE_OK) {                           
    gtk_entry_set_text (GTK_ENTRY (entry1), directory.c_str ());
  }
}


void FopDialog::on_button2_clicked (GtkButton * button, gpointer user_data)
{
  ((FopDialog *) user_data)->on_button2 ();
}


void FopDialog::on_button2 ()
{
  GtkWidget *file_selector;
  /* Create the selector */
  file_selector = gtk_file_selection_new ("Please choose a directory");
  // Prevent the user from selecting a file.
  gtk_widget_hide (GTK_FILE_SELECTION (file_selector)->file_list);
  gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (file_selector));
  string directory = gtk_entry_get_text (GTK_ENTRY (entry2));
  directory.append ("/");
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_selector), directory.c_str());
  // Display that dialog
  gtk_widget_show (file_selector);
  int result = gtk_dialog_run (GTK_DIALOG (file_selector));
  directory = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
  gtk_widget_destroy (file_selector);
  // Handle the response:
  if (result == GTK_RESPONSE_OK) {                           
    gtk_entry_set_text (GTK_ENTRY (entry2), directory.c_str ());
  }
}
