/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogstylesheet.h"
#include "utilities.h"
#include "stylesheetutils.h"
#include "dialogentry.h"
#include <set>
#include "project.h"
#include "projects.h"
#include "directories.h"
#include "dialogentry3.h"
#include "usfm.h"


StylesheetDialog::StylesheetDialog (const string& name)
{
  // Save variables.
  sheetname = name;

  // Copy stylesheet to a temporal one where we will work on.
  stylesheet_copy (name, TEMPORAL_STYLESHEET);
  
  GSList *radiobutton_id_group = NULL;
  GSList *radiobutton_left_group = NULL;

  stylesheetdialog = gtk_dialog_new ();
  string s;
  s = "Stylesheet " + name;
  gtk_window_set_title (GTK_WINDOW (stylesheetdialog), s.c_str());
  gtk_window_set_position (GTK_WINDOW (stylesheetdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (stylesheetdialog), TRUE);
  gtk_window_set_destroy_with_parent (GTK_WINDOW (stylesheetdialog), TRUE);
  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (stylesheetdialog), TRUE);
  gtk_window_set_skip_pager_hint (GTK_WINDOW (stylesheetdialog), TRUE);
  gtk_window_set_type_hint (GTK_WINDOW (stylesheetdialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (stylesheetdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  vbox2 = gtk_vbox_new (FALSE, 6);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), vbox2, TRUE, TRUE, 0);

  hbox1 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox1), 2);

  label6 = gtk_label_new_with_mnemonic ("_Style");
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (hbox1), label6, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label6), 0, 0.5);

  combo_style = gtk_combo_new ();
  gtk_widget_show (combo_style);
  gtk_box_pack_start (GTK_BOX (hbox1), combo_style, TRUE, TRUE, 0);

  combo_entry_style = GTK_COMBO (combo_style)->entry;
  gtk_widget_show (combo_entry_style);
  gtk_widget_set_size_request (combo_entry_style, 200, -1);
  gtk_editable_set_editable (GTK_EDITABLE (combo_entry_style), FALSE);

  button_new = gtk_button_new_from_stock ("gtk-new");
  gtk_widget_show (button_new);
  gtk_box_pack_start (GTK_BOX (hbox1), button_new, FALSE, FALSE, 0);

  button_delete = gtk_button_new_from_stock ("gtk-delete");
  gtk_widget_show (button_delete);
  gtk_box_pack_start (GTK_BOX (hbox1), button_delete, FALSE, FALSE, 0);

  infolabel = gtk_label_new ("");
  gtk_widget_show (infolabel);
  gtk_box_pack_start (GTK_BOX (vbox2), infolabel, FALSE, FALSE, 0);
  gtk_widget_set_size_request (infolabel, -1, 80);
  gtk_label_set_line_wrap (GTK_LABEL (infolabel), TRUE);
  gtk_misc_set_alignment (GTK_MISC (infolabel), 0, 0);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox2), hseparator1, TRUE, TRUE, 0);

  label21 = gtk_label_new ("This style:");
  gtk_widget_show (label21);
  gtk_box_pack_start (GTK_BOX (vbox2), label21, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label21), 0, 0.5);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_box_pack_start (GTK_BOX (vbox2), vbox3, TRUE, TRUE, 0);

  radiobutton_id = gtk_radio_button_new_with_mnemonic (NULL, "is an identifier");
  gtk_widget_show (radiobutton_id);
  gtk_box_pack_start (GTK_BOX (vbox3), radiobutton_id, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_id), radiobutton_id_group);
  radiobutton_id_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_id));

  radiobutton_comment = gtk_radio_button_new_with_mnemonic (NULL, "is a comment");
  gtk_widget_show (radiobutton_comment);
  gtk_box_pack_start (GTK_BOX (vbox3), radiobutton_comment, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_comment), radiobutton_id_group);
  radiobutton_id_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_comment));

  radiobutton_runningheader = gtk_radio_button_new_with_mnemonic (NULL, "is a running header");
  gtk_widget_show (radiobutton_runningheader);
  gtk_box_pack_start (GTK_BOX (vbox3), radiobutton_runningheader, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_runningheader), radiobutton_id_group);
  radiobutton_id_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_runningheader));

  radiobutton_paragraph = gtk_radio_button_new_with_mnemonic (NULL, "starts a new paragraph");
  gtk_widget_show (radiobutton_paragraph);
  gtk_box_pack_start (GTK_BOX (vbox3), radiobutton_paragraph, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_paragraph), radiobutton_id_group);
  radiobutton_id_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_paragraph));

  radiobutton_inline = gtk_radio_button_new_with_mnemonic (NULL, "is inline text");
  gtk_widget_show (radiobutton_inline);
  gtk_box_pack_start (GTK_BOX (vbox3), radiobutton_inline, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_inline), radiobutton_id_group);
  radiobutton_id_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_inline));

  radiobutton_chapter = gtk_radio_button_new_with_mnemonic (NULL, "is a chapter number");
  gtk_widget_show (radiobutton_chapter);
  gtk_box_pack_start (GTK_BOX (vbox3), radiobutton_chapter, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_chapter), radiobutton_id_group);
  radiobutton_id_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_chapter));

  radiobutton_verse = gtk_radio_button_new_with_mnemonic (NULL, "is a verse number");
  gtk_widget_show (radiobutton_verse);
  gtk_box_pack_start (GTK_BOX (vbox3), radiobutton_verse, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_verse), radiobutton_id_group);
  radiobutton_id_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_verse));

  hseparator2 = gtk_hseparator_new ();
  gtk_widget_show (hseparator2);
  gtk_box_pack_start (GTK_BOX (vbox2), hseparator2, TRUE, TRUE, 0);

  notebookfontsize = gtk_notebook_new ();
  gtk_widget_show (notebookfontsize);
  gtk_box_pack_start (GTK_BOX (vbox2), notebookfontsize, TRUE, TRUE, 0);
  GTK_WIDGET_UNSET_FLAGS (notebookfontsize, GTK_CAN_FOCUS);
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebookfontsize), FALSE);
  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebookfontsize), FALSE);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (notebookfontsize), hbox2);

  label8 = gtk_label_new ("Font size");
  gtk_widget_show (label8);
  gtk_box_pack_start (GTK_BOX (hbox2), label8, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label8), 4, 0);

  spinbuttonfontsize_adj = gtk_adjustment_new (12, 5, 60, 0.1, 1, 1);
  spinbuttonfontsize = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonfontsize_adj), 1, 1);
  gtk_widget_show (spinbuttonfontsize);
  gtk_box_pack_start (GTK_BOX (hbox2), spinbuttonfontsize, FALSE, FALSE, 0);

  label9 = gtk_label_new ("points");
  gtk_widget_show (label9);
  gtk_box_pack_start (GTK_BOX (hbox2), label9, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label9), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label9), 4, 0);

  label22 = gtk_label_new ("label22");
  gtk_widget_show (label22);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookfontsize), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookfontsize), 0), label22);

  empty_notebook_page = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (empty_notebook_page);
  gtk_container_add (GTK_CONTAINER (notebookfontsize), empty_notebook_page);

  label23 = gtk_label_new ("label23");
  gtk_widget_show (label23);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebookfontsize), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebookfontsize), 1), label23);

  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox5, TRUE, TRUE, 0);

  notebook_italics_bold = gtk_notebook_new ();
  gtk_widget_show (notebook_italics_bold);
  gtk_box_pack_start (GTK_BOX (hbox5), notebook_italics_bold, FALSE, TRUE, 0);
  GTK_WIDGET_UNSET_FLAGS (notebook_italics_bold, GTK_CAN_FOCUS);
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook_italics_bold), FALSE);
  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook_italics_bold), FALSE);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox3);
  gtk_container_add (GTK_CONTAINER (notebook_italics_bold), hbox3);

  checkbutton_italics = gtk_check_button_new_with_mnemonic ("Italics");
  gtk_widget_show (checkbutton_italics);
  gtk_box_pack_start (GTK_BOX (hbox3), checkbutton_italics, FALSE, FALSE, 0);

  checkbutton_bold = gtk_check_button_new_with_mnemonic ("Bold");
  gtk_widget_show (checkbutton_bold);
  gtk_box_pack_start (GTK_BOX (hbox3), checkbutton_bold, FALSE, FALSE, 0);

  label24 = gtk_label_new ("label24");
  gtk_widget_show (label24);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_italics_bold), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_italics_bold), 0), label24);

  empty_notebook_page = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (empty_notebook_page);
  gtk_container_add (GTK_CONTAINER (notebook_italics_bold), empty_notebook_page);

  label25 = gtk_label_new ("label25");
  gtk_widget_show (label25);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_italics_bold), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_italics_bold), 1), label25);

  notebook_superscript = gtk_notebook_new ();
  gtk_widget_show (notebook_superscript);
  gtk_box_pack_start (GTK_BOX (hbox5), notebook_superscript, TRUE, TRUE, 0);
  GTK_WIDGET_UNSET_FLAGS (notebook_superscript, GTK_CAN_FOCUS);
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook_superscript), FALSE);
  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook_superscript), FALSE);

  checkbutton_superscript = gtk_check_button_new_with_mnemonic ("Superscript");
  gtk_widget_show (checkbutton_superscript);
  gtk_container_add (GTK_CONTAINER (notebook_superscript), checkbutton_superscript);

  label28 = gtk_label_new ("label28");
  gtk_widget_show (label28);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_superscript), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_superscript), 0), label28);

  empty_notebook_page = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (empty_notebook_page);
  gtk_container_add (GTK_CONTAINER (notebook_superscript), empty_notebook_page);

  label29 = gtk_label_new ("label29");
  gtk_widget_show (label29);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_superscript), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_superscript), 1), label29);

  notebook_paragraph = gtk_notebook_new ();
  gtk_widget_show (notebook_paragraph);
  gtk_box_pack_start (GTK_BOX (vbox2), notebook_paragraph, TRUE, TRUE, 0);
  GTK_WIDGET_UNSET_FLAGS (notebook_paragraph, GTK_CAN_FOCUS);
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook_paragraph), FALSE);
  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook_paragraph), FALSE);

  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox4);
  gtk_container_add (GTK_CONTAINER (notebook_paragraph), vbox4);

  hbox4 = gtk_hbox_new (FALSE, 10);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox4, TRUE, TRUE, 0);

  label10 = gtk_label_new ("Text alignment");
  gtk_widget_show (label10);
  gtk_box_pack_start (GTK_BOX (hbox4), label10, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label10), 0, 0.5);

  radiobutton_left = gtk_radio_button_new_with_mnemonic (NULL, "Left");
  gtk_widget_show (radiobutton_left);
  gtk_box_pack_start (GTK_BOX (hbox4), radiobutton_left, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_left), radiobutton_left_group);
  radiobutton_left_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_left));

  radiobutton_center = gtk_radio_button_new_with_mnemonic (NULL, "Center");
  gtk_widget_show (radiobutton_center);
  gtk_box_pack_start (GTK_BOX (hbox4), radiobutton_center, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_center), radiobutton_left_group);
  radiobutton_left_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_center));

  radiobutton_right = gtk_radio_button_new_with_mnemonic (NULL, "Right");
  gtk_widget_show (radiobutton_right);
  gtk_box_pack_start (GTK_BOX (hbox4), radiobutton_right, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_right), radiobutton_left_group);
  radiobutton_left_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_right));

  radiobutton_full = gtk_radio_button_new_with_mnemonic (NULL, "Justified");
  gtk_widget_show (radiobutton_full);
  gtk_box_pack_start (GTK_BOX (hbox4), radiobutton_full, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_full), radiobutton_left_group);
  radiobutton_left_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_full));

  table1 = gtk_table_new (5, 4, FALSE);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox4), table1, TRUE, TRUE, 0);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 4);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 4);

  label11 = gtk_label_new ("Space before paragraph");
  gtk_widget_show (label11);
  gtk_table_attach (GTK_TABLE (table1), label11, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label11), 1, 0.5);

  label12 = gtk_label_new ("Space after paragraph");
  gtk_widget_show (label12);
  gtk_table_attach (GTK_TABLE (table1), label12, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label12), 1, 0.5);

  label13 = gtk_label_new ("Left margin");
  gtk_widget_show (label13);
  gtk_table_attach (GTK_TABLE (table1), label13, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label13), 1, 0.5);

  label14 = gtk_label_new ("Right margin");
  gtk_widget_show (label14);
  gtk_table_attach (GTK_TABLE (table1), label14, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label14), 1, 0.5);

  label15 = gtk_label_new ("First line indent");
  gtk_widget_show (label15);
  gtk_table_attach (GTK_TABLE (table1), label15, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label15), 1, 0.5);

  label16 = gtk_label_new ("mm");
  gtk_widget_show (label16);
  gtk_table_attach (GTK_TABLE (table1), label16, 2, 3, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label16), 0, 0.5);

  label17 = gtk_label_new ("mm");
  gtk_widget_show (label17);
  gtk_table_attach (GTK_TABLE (table1), label17, 2, 3, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label17), 0, 0.5);

  label18 = gtk_label_new ("mm");
  gtk_widget_show (label18);
  gtk_table_attach (GTK_TABLE (table1), label18, 2, 3, 2, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label18), 0, 0.5);

  label19 = gtk_label_new ("mm");
  gtk_widget_show (label19);
  gtk_table_attach (GTK_TABLE (table1), label19, 2, 3, 3, 4,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label19), 0, 0.5);

  label20 = gtk_label_new ("mm");
  gtk_widget_show (label20);
  gtk_table_attach (GTK_TABLE (table1), label20, 2, 3, 4, 5,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label20), 0, 0.5);

  spinbutton_first_adj = gtk_adjustment_new (0, 0, 100, 0.1, 1, 1);
  spinbutton_first = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_first_adj), 1, 1);
  gtk_widget_show (spinbutton_first);
  gtk_table_attach (GTK_TABLE (table1), spinbutton_first, 1, 2, 4, 5,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  spinbutton_before_adj = gtk_adjustment_new (0, 0, 100, 0.1, 1, 1);
  spinbutton_before = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_before_adj), 1, 1);
  gtk_widget_show (spinbutton_before);
  gtk_table_attach (GTK_TABLE (table1), spinbutton_before, 1, 2, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  spinbutton_after_adj = gtk_adjustment_new (0, 0, 100, 0.1, 1, 1);
  spinbutton_after = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_after_adj), 1, 1);
  gtk_widget_show (spinbutton_after);
  gtk_table_attach (GTK_TABLE (table1), spinbutton_after, 1, 2, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  spinbutton_left_adj = gtk_adjustment_new (0, 0, 100, 0, 1, 1);
  spinbutton_left = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_left_adj), 1, 1);
  gtk_widget_show (spinbutton_left);
  gtk_table_attach (GTK_TABLE (table1), spinbutton_left, 1, 2, 2, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  spinbutton_right_adj = gtk_adjustment_new (0, 0, 100, 0.1, 1, 1);
  spinbutton_right = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_right_adj), 1, 1);
  gtk_widget_show (spinbutton_right);
  gtk_table_attach (GTK_TABLE (table1), spinbutton_right, 1, 2, 3, 4,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  checkbutton_span = gtk_check_button_new_with_mnemonic ("Spans the two columns");
  gtk_widget_show (checkbutton_span);
  gtk_box_pack_start (GTK_BOX (vbox4), checkbutton_span, FALSE, FALSE, 0);

  label26 = gtk_label_new ("label26");
  gtk_widget_show (label26);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_paragraph), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_paragraph), 0), label26);

  empty_notebook_page = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (empty_notebook_page);
  gtk_container_add (GTK_CONTAINER (notebook_paragraph), empty_notebook_page);

  label27 = gtk_label_new ("label27");
  gtk_widget_show (label27);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_paragraph), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_paragraph), 1), label27);

  dialog_action_area1 = GTK_DIALOG (stylesheetdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (stylesheetdialog), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (stylesheetdialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) combo_entry_style, "changed",
                    G_CALLBACK (on_combo_entry_style_changed),
                    gpointer(this));
  g_signal_connect ((gpointer) button_new, "clicked",
                    G_CALLBACK (on_button_new_clicked),
                    gpointer(this));
  g_signal_connect ((gpointer) button_delete, "clicked",
                    G_CALLBACK (on_button_delete_clicked),
                    gpointer(this));
  g_signal_connect ((gpointer) okbutton, "clicked",
                    G_CALLBACK (on_okbutton_clicked),
                    gpointer(this));
  g_signal_connect ((gpointer) radiobutton_id, "toggled",
                    G_CALLBACK (on_style_type_toggled),
                    gpointer(this));
  g_signal_connect ((gpointer) radiobutton_comment, "toggled",
                    G_CALLBACK (on_style_type_toggled),
                    gpointer(this));
  g_signal_connect ((gpointer) radiobutton_runningheader, "toggled",
                    G_CALLBACK (on_style_type_toggled),
                    gpointer(this));
  g_signal_connect ((gpointer) radiobutton_paragraph, "toggled",
                    G_CALLBACK (on_style_type_toggled),
                    gpointer(this));
  g_signal_connect ((gpointer) radiobutton_inline, "toggled",
                    G_CALLBACK (on_style_type_toggled),
                    gpointer(this));
  g_signal_connect ((gpointer) radiobutton_chapter, "toggled",
                    G_CALLBACK (on_style_type_toggled),
                    gpointer(this));
  g_signal_connect ((gpointer) radiobutton_verse, "toggled",
                    G_CALLBACK (on_style_type_toggled),
                    gpointer(this));

  gtk_label_set_mnemonic_widget (GTK_LABEL (label6), combo_entry_style);

  gtk_widget_grab_default (okbutton);
  
  // Get all the styles in the GUI.
  load_styles ("");

}


StylesheetDialog::~StylesheetDialog ()
{
  gtk_widget_destroy (stylesheetdialog);
}


int StylesheetDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (stylesheetdialog));
}


void StylesheetDialog::on_okbutton_clicked (GtkButton *button, gpointer user_data)
{
  ((StylesheetDialog *) user_data)->on_ok ();
}


void StylesheetDialog::on_ok()
{
  // Save current style, if there was a change.
  save_style ();
  // Copy the temporal stylesheet back to the one we're editing.
  stylesheet_delete (sheetname);
  stylesheet_copy (TEMPORAL_STYLESHEET, sheetname);
}


void StylesheetDialog::on_combo_entry_style_changed (GtkEditable *editable, gpointer user_data)
{
  ((StylesheetDialog *) user_data)->on_combo_entry_style ();
}


void StylesheetDialog::on_combo_entry_style ()
{
  // Save possibly changed previous style.
  save_style ();
  // Get the name of the style we now display.
  string name;
  name = gtk_entry_get_text (GTK_ENTRY (combo_entry_style));
  // Get all styles, and pick out the one with the right name.
  string stylesheet = TEMPORAL_STYLESHEET;
  vector<Style> styles;
  stylesheet_get_styles (stylesheet, styles);
  for (unsigned int i = 0; i < styles.size(); i++) {
    if (name == styles[i].name) {
      // Store marker.
      currentmarker = styles[i].marker;
      // Display information about this style.
      string info;
      info = "Marker \\" + currentmarker + "\n" + styles[i].info;
      gtk_label_set_text (GTK_LABEL (infolabel), info.c_str());
      // Set publication information.
      switch (styles[i].type) {
        case stIdentifier      : 
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_id), true);
          break;
        case stComment         : 
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_comment), true);
          break;
        case stRunningHeader   :
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_runningheader), true);
          break;
        case stStartsParagraph :
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_paragraph), true);
          break;
        case stInlineText      :
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_inline), true);
          break;
        case stChapterNumber   :
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_chapter), true);
          break;
        case stVerseNumber     :
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_verse), true);
          break;
      }
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbuttonfontsize), styles[i].fontsize);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_italics), styles[i].italics);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_bold), styles[i].bold);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_superscript), styles[i].superscript);
      set_justification (styles[i].justification);
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton_before), styles[i].spacebefore);
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton_after), styles[i].spaceafter);
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton_left), styles[i].leftmargin);
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton_right), styles[i].rightmargin);
      gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton_first), styles[i].firstlineindent);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_span), styles[i].spancolumns);
      // Is this a fixed style?
      gtk_widget_set_sensitive (button_delete, !usfm_is_fixed (currentmarker));
    }
  }
}


void StylesheetDialog::on_button_new_clicked (GtkButton *button, gpointer user_data)
{
  ((StylesheetDialog *) user_data)->on_new ();  
}


void StylesheetDialog::on_new()
{
  Entry3Dialog dialog ("New style", "_Marker", "", "_Name of the marker", "", "_Description of the marker", "");
  dialog.run();
  // Check that the marker or the name for the style do not already exist.
  string stylesheet = TEMPORAL_STYLESHEET;
  vector<Style> styles;
  bool marker_exists = false;
  bool name_exists = false;
  stylesheet_get_styles (stylesheet, styles);
  for (unsigned int i = 0; i < styles.size(); i++) {
    if (dialog.entered_value1 == styles[i].marker)
      marker_exists = true;
    if (dialog.entered_value2 == styles[i].name)
      name_exists = true;
  }
  if (marker_exists) {
    GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (stylesheetdialog), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "This marker already exists in the stylesheet");
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
  }
  if (name_exists) {
    GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (stylesheetdialog), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "A style with this name already exists in the stylesheet");
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
  }
  if (!marker_exists && !name_exists) {
    stylesheet_new_style (TEMPORAL_STYLESHEET, dialog.entered_value1, dialog.entered_value2, dialog.entered_value3);
    load_styles (dialog.entered_value2);
  }
}


void StylesheetDialog::on_button_delete_clicked (GtkButton *button, gpointer user_data)
{
  ((StylesheetDialog *) user_data)->on_delete ();
}


void StylesheetDialog::on_delete()
{
  // Delete style.
  stylesheet_delete_style (TEMPORAL_STYLESHEET, currentmarker);
  // Prevent automatic creation of style just deleted.
  currentmarker.clear();
  // Load new styles in GUI.
  load_styles ("");
}


void StylesheetDialog::on_style_type_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
  ((StylesheetDialog *) user_data)->on_style_type ();
}


void StylesheetDialog::on_style_type ()
{
  // Get the type of the style.
  bool h = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_runningheader));
  bool p = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_paragraph));
  bool text = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_inline));
  bool c = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_chapter));
  bool v = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_verse));
  // Which tab of the notebook to display.
  int tab = 0;  
  // Show or hide fontsize.
  if (h || p || text || c || v) {
    tab = 0;
  } else {
    tab = 1;
  }
  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebookfontsize), tab);
  // Show or hide italics, bold tab.
  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook_italics_bold), tab);
  // Show or hide superscript tab.
  if (text || v) {
    tab = 0;
  } else {
    tab = 1;
  }
  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook_superscript), tab);
  // Show/hide paragraph data.
  if (p) {
    tab = 0;
  } else {
    tab = 1;
  }
  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook_paragraph), tab);
}


void StylesheetDialog::load_styles (const string& select)
// Loads the styles in the combo box, and selects one.
{
  string stylesheet = TEMPORAL_STYLESHEET;
  vector<Style> styles;
  stylesheet_get_styles (stylesheet, styles);
  GList *combo_style_items = NULL;
  for (unsigned int i = 0; i < styles.size(); i++)
    combo_style_items = g_list_append (combo_style_items, (gpointer) styles[i].name.c_str());
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_style), combo_style_items);
  g_list_free (combo_style_items);
  if (select.empty()) {
    // None selected, select first in list, if available.
    if (styles.size() > 0)
      gtk_entry_set_text (GTK_ENTRY (combo_entry_style), styles[0].name.c_str());
  } else {
    // Select this one.
    gtk_entry_set_text (GTK_ENTRY (combo_entry_style), select.c_str());
  }
}


void StylesheetDialog::save_style ()
{
  if (!currentmarker.empty()) {
    StyleType type;
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_id))) {
      type = stIdentifier;
    }
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_comment))) {
      type = stComment;
    }
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_runningheader))) {
      type = stRunningHeader;
    }
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_paragraph))) {
      type = stStartsParagraph;
    }
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_inline))) {
      type = stInlineText;
    }
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_chapter))) {
      type = stChapterNumber;
    }
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_verse))) {
      type = stVerseNumber;
    }
    stylesheet_save_style (TEMPORAL_STYLESHEET, currentmarker,
      type,
      gtk_spin_button_get_value (GTK_SPIN_BUTTON (spinbuttonfontsize)),
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_italics)),
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_bold)),
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_superscript)),
      get_justification(),
      gtk_spin_button_get_value (GTK_SPIN_BUTTON (spinbutton_before)),
      gtk_spin_button_get_value (GTK_SPIN_BUTTON (spinbutton_after)),
      gtk_spin_button_get_value (GTK_SPIN_BUTTON (spinbutton_left)),
      gtk_spin_button_get_value (GTK_SPIN_BUTTON (spinbutton_right)),
      gtk_spin_button_get_value (GTK_SPIN_BUTTON (spinbutton_first)),
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_span))
    );
    currentmarker.clear();
  }
}


#define LEFT "left"
#define CENTER "center"
#define RIGHT "right"
#define JUSTIFIED "justify"


void StylesheetDialog::set_justification (const string& justification)
// Sets the GUI.
{
  if (justification == CENTER) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_center), true);
  } else if (justification == RIGHT) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_right), true);
  } else if (justification == JUSTIFIED) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_full), true);
  } else {
    // Defaults to left.
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_left), true);
  }
}


string StylesheetDialog::get_justification ()
// Get the string that indicates the justification, from the GUI.
{
  string result;
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_center))) {
    result = CENTER;
  } else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_right))) {
    result = RIGHT;
  } else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_full))) {
    result = JUSTIFIED;
  } else {
    // Defaults to left.
    result = LEFT;
  }
  return result;
}
