/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogstylesheets.h"
#include "utilities.h"
#include "stylesheetutils.h"
#include "dialogentry.h"
#include <set>
#include "project.h"
#include "projects.h"
#include "directories.h"
#include "dialogstylesheet.h"


StylesheetsDialog::StylesheetsDialog (Configuration * configuration)
{
  
  myconfig = configuration;
  
  stylesheetsdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (stylesheetsdialog), "Stylesheets");
  gtk_window_set_position (GTK_WINDOW (stylesheetsdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (stylesheetsdialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (stylesheetsdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  combo1 = gtk_combo_new ();
  gtk_widget_show (combo1);
  gtk_box_pack_start (GTK_BOX (vbox1), combo1, FALSE, FALSE, 0);

  combo_entry1 = GTK_COMBO (combo1)->entry;
  gtk_widget_show (combo_entry1);
  gtk_editable_set_editable (GTK_EDITABLE (combo_entry1), FALSE);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 2);

  newbutton = gtk_button_new_from_stock ("gtk-new");
  gtk_widget_show (newbutton);
  gtk_box_pack_start (GTK_BOX (hbox1), newbutton, FALSE, FALSE, 0);
  gtk_widget_set_size_request (newbutton, 90, -1);

  propertiesbutton = gtk_button_new_from_stock ("gtk-properties");
  gtk_widget_show (propertiesbutton);
  gtk_box_pack_start (GTK_BOX (hbox1), propertiesbutton, FALSE, FALSE, 0);
  gtk_widget_set_size_request (propertiesbutton, 90, -1);

  deletebutton = gtk_button_new_from_stock ("gtk-delete");
  gtk_widget_show (deletebutton);
  gtk_box_pack_start (GTK_BOX (hbox1), deletebutton, FALSE, FALSE, 0);
  gtk_widget_set_size_request (deletebutton, 90, -1);

  dialog_action_area1 = GTK_DIALOG (stylesheetsdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  closebutton1 = gtk_button_new_from_stock ("gtk-close");
  gtk_widget_show (closebutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (stylesheetsdialog), closebutton1, GTK_RESPONSE_CLOSE);
  GTK_WIDGET_SET_FLAGS (closebutton1, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) newbutton, "clicked", G_CALLBACK (on_newbutton_clicked), gpointer(this));
  g_signal_connect ((gpointer) propertiesbutton, "clicked",  G_CALLBACK (on_propertiesbutton_clicked), gpointer(this));
  g_signal_connect ((gpointer) deletebutton, "clicked", G_CALLBACK (on_deletebutton_clicked), gpointer(this));

  gtk_widget_grab_focus (closebutton1);
  gtk_widget_grab_default (closebutton1);
  
  Project project (configuration);
  set_gui (project.stylesheet);
}


StylesheetsDialog::~StylesheetsDialog ()
{
  gtk_widget_destroy (stylesheetsdialog);
}


int StylesheetsDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (stylesheetsdialog));
}


void StylesheetsDialog::on_newbutton_clicked (GtkButton *button, gpointer user_data)
{
  ((StylesheetsDialog *) user_data)->on_new ();
}


void StylesheetsDialog::on_new ()
{
  // Get the name for the new stylesheet.
  string name;
  EntryDialog dialog ("New stylesheet", "Enter the name for the new stylesheet", "");
  if (dialog.run () == GTK_RESPONSE_OK) {
    name = dialog.entered_value;
    // If it exists already, do not create a new one.
    vector<string> sheets;
    stylesheet_get_ones_available (sheets);
    set<string> existing_sheets (sheets.begin(), sheets.end());
    if (existing_sheets.find (name) == existing_sheets.end()) {
      stylesheet_create_new (name);
      set_gui(name);
    } else {
      GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (stylesheetsdialog), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "A stylesheet with this name already exists.");
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);      
    }
  }
}


void StylesheetsDialog::on_propertiesbutton_clicked (GtkButton *button, gpointer user_data)
{
  ((StylesheetsDialog *) user_data)->on_properties ();
}


void StylesheetsDialog::on_properties ()
{
  string name = gtk_entry_get_text (GTK_ENTRY (combo_entry1));
  StylesheetDialog dialog (name);
  dialog.run();
  // Remove temporal stylesheet.
  stylesheet_delete (TEMPORAL_STYLESHEET);
}


void StylesheetsDialog::on_deletebutton_clicked (GtkButton *button, gpointer user_data)
{
  ((StylesheetsDialog *) user_data)->on_delete ();
}


void StylesheetsDialog::on_delete ()
{
  // Delete the stylesheet if it is not used in any project.
  string name = gtk_entry_get_text (GTK_ENTRY (combo_entry1));
  set<ustring> sheets_in_use;
  ustring current_project = myconfig->project;
  Projects projects (directories_get_projects ());
  for (unsigned int i = 0; i < projects.projects.size(); i++) {
    myconfig->project = projects.projects[i];
    Project project (myconfig);
    sheets_in_use.insert (project.stylesheet);
  }
  myconfig->project = current_project;
  if (sheets_in_use.find(name) == sheets_in_use.end()) {
    // Stylesheet not in use: delete it.
    stylesheet_delete (name);
    set_gui ("");
  } else {
    // Stylesheet in use: keep it.
    GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (stylesheetsdialog), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "The stylesheet cannot be deleted, because it is in use by a project.");
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
  }
}


void StylesheetsDialog::set_gui (const string& select)
{
  // Display available stylesheets.
  vector<string> stylesheets;
  stylesheet_get_ones_available (stylesheets);
  GList *combo1_items = NULL;
  for (unsigned int i = 0; i < stylesheets.size(); i++) {
    combo1_items = g_list_append (combo1_items, (gpointer) stylesheets[i].c_str());
  }
  gtk_combo_set_popdown_strings (GTK_COMBO (combo1), combo1_items);
  g_list_free (combo1_items);
  if (!select.empty())
    gtk_entry_set_text (GTK_ENTRY (combo_entry1), select.c_str());
  else {
    if (stylesheets.size() > 0)
      gtk_entry_set_text (GTK_ENTRY (combo_entry1), stylesheets[0].c_str());
  }
}
