/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "export_utils.h"
#include "scripture.h"
#include "book.h"
#include "bible.h"
#include "usfm.h"
#include "usfmtools.h"


void export_to_bibleworks (Configuration * configuration, Usfm * usfm,
                           GtkWidget * progressbar, GtkWidget * notebook)
/*
Exports a whole project to a file that is fit for being imported by the 
BibleWorks Version Database Compiler.
This done in this manner:
- If an \id line is found, use its ID to get the name of the book in BibleWorks.
- If an chapter tag if found, use its number.
- As soon as an ID is found, put the current chapter at one.
- If a verse is found, add its number.
- Text of tags known not to be printable is thrown out.
- Any other tag's text gets added to the text of the verse.
Yes, this is a bit rough, I know...
*/
{
  // Notebook shows progress bar.
  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 1);
  // Get the filename where the data to save to.
  GtkWidget *file_selector;
  file_selector = gtk_file_selection_new ("Please choose a file");
  gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (file_selector));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_selector), configuration->export_to_bibleworks_filename.c_str ());
  gtk_widget_show (file_selector);
  int result = gtk_dialog_run (GTK_DIALOG (file_selector));
  string filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
  gtk_widget_destroy (file_selector);
  if (result == GTK_RESPONSE_OK) {
    // Store (new) filename.
    configuration->export_to_bibleworks_filename = filename;
    // Start process.
    try
    {
      // Write to outputfile.
      WriteText wt (filename);
      // Open the Scripture and go through it.
      Scripture scripture (configuration->project);
      for (unsigned int bk = 0; bk < scripture.books.size(); bk++) {
        // Progress information.
        double percentage;
        percentage = double ((bk + 1)) / scripture.books.size();
        gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar), percentage);
        while (gtk_events_pending ()) gtk_main_iteration ();
        // Get the abbreviation to be used.
        string abbreviation;
        unsigned index = english_id_to_index (scripture.books[bk]);
        abbreviation = index_to_bibleworks_id (index);
        // Open the book
        Book book (scripture.paths[bk], true, false, usfm);
        // Start with chapter 1.
        string chapter = "1";
        // Go through the book.
        for (unsigned int i = 0; i < book.cleaned_lines.size(); i++) {
          ustring marker = usfm_extract_marker (book.cleaned_lines[i]);
          // Skip id line.
          if (usfm->is_identifier (marker))
            continue;
          // Skip running header.
          if (usfm->is_running_header (marker))
            continue;
          // Save chapter number.
          if (usfm->is_chapter_number(marker)) {
            chapter = number_in_string (book.cleaned_lines[i]);
            continue;
          }
          // If verse found, output new line, and its text.
          if (usfm->is_verse_number(marker)) {
            wt.out << endl << abbreviation << " " << chapter << ":" << book.cleaned_lines[i];
            continue;
          }
          // Any other marker, known or unknown, output its text.
          wt.out << " " << book.cleaned_lines[i];
        }
      }
    }
    catch (exception & ex)
    {
      cerr << "Converting to BibleWorks: " << ex.what () << endl;
    }
  }
  // Notebook hides progress bar.
  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar), 0);
  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 0);
}


/*
      // Get chapter of the original.
      vector <ustring> originallines;
      originalbook.get_chapter (ic, originallines);
      // Get chapter of the second.
      vector <ustring> secondlines;
      secondbook.get_chapter (ic, secondlines);
      // If both chapters are the same, continue to the next one.
      ustring originalbook_chapter_contents;
      for (unsigned int i = 0; i < originallines.size(); i++)
        originalbook_chapter_contents.append (originallines[i]);
      ustring secondbook_chapter_contents;
      for (unsigned int i = 0; i < secondlines.size(); i++)
        secondbook_chapter_contents.append(secondlines[i]);
      if (originalbook_chapter_contents == secondbook_chapter_contents)
        continue;
      // At this stage the chapters are different. Compare them more thoroughly.
      vector <ustring> outputlines;
      outputbook.get_chapter (ic, outputlines);
      compare_chapter (originallines, secondlines, outputlines);
      outputbook.set_chapter (outputlines);
      outputbook.save();
*/
