/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include <libgen.h>
#include <glib.h>
#include <config.h>
#include "fop.h"
#include "java.h"


/*
At first passivetex, xmlpdftex, was used, but printing a full project
caused an overflow of the capacity of TeX.
For this reason the switch to FOP was made.
A disadvantage of FOP is that it is written in Java, and therefore is slow
and difficult to install, but anyway, there was no better free
xsl-fo processor available than FOP.
As bibledit aims to produce good quality printed output, we will use FOP.
Until a better renderer becomes available.
*/


string fop_command ()
{
  // Return the command to run script fop, if found.
  string returnvalue;
  if (system ("which fop") == 0)
    returnvalue = "fop";
  return returnvalue;
}


string fop_sh_command (Configuration * configuration)
{
  // Return the command to run script fop.sh, if found.
  string returnvalue;
  string command;
  command = "which ";
  string fop_sh = g_build_filename (configuration->fop_home.c_str(), "fop.sh", NULL);
  command.append (fop_sh);
  if (system (command.c_str()) != 0)
    fop_sh.clear();
  return fop_sh;
}


bool fop_present (GtkWidget * window, Configuration * configuration)
// Returns whether a script to run fop can be found.
// Gives message if not found.
{
  bool present = false;
  // Try the fop script first.  
  if (!fop_command().empty())
    present = true;
  // Else look for the fop.sh script.
  else if (!fop_sh_command(configuration).empty())
    present = true;
  
  if (!present) {
    string message;
    message = "FOP was not found.\n";
    message.append ("FOP is a \"Formatting Objects Processor\".\n");
    message.append ("It is required for printing.\n");
    message.append ("Install FOP first.\n");
    message.append ("For more information see menu Preferences->FOP.");
    GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (window), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, message.c_str());
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
  }
  return present;
}


int fop_convert_to_pdf (Configuration * configuration, const string& xslfo, const string& pdf)
// Converts file "xslfo" to file "pdf", and returns the result.
{
  // Check the Java environment.
  string command;
  if (java_home_environment_variable().empty()) {
    command.append ("JAVA_HOME=");
    command.append (configuration->java_home);
    command.append ("; export JAVA_HOME; ");
  }
  // Try script fop first.
  string fop;
  fop = fop_command();
  if (!fop.empty()) {
    command.append (fop);
  } else {
    // Next try script fop.sh.
    fop = fop_sh_command(configuration);
    if (!fop.empty()) {
      command.append (fop);
    }
  }
  command.append (" '");
  command.append (xslfo);
  command.append ("' '");
  command.append (pdf);
  command.append ("'");
  int result;
  result = system (command.c_str());
  return result;
}
