/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "project.h"
#include "utilities.h"
#include "config.h"
#include <glib.h>
#include "style.h"


Style::Style (const string& directory, const string& style, bool write)
{
  // Save variabele.
  mywrite = write;
  marker = style;
  
  // Read the style file, if there is any. Add the bits that are missing.
  {
    gchar *name;
    name = g_build_filename (directory.c_str(), style.c_str(), NULL);
    filename = name;
    g_free (name);
  }
  ReadText rc (filename, true);
  for (unsigned int i = 0; i <= 1000; i++) {
    switch (i) {
      case 2:
      {
        if (i < rc.lines.size ()) {
          name = rc.lines[i];
        }
        else {
          name = "Marker";
        }
        break;
      }
      case 3:
      {
        if (i < rc.lines.size ()) {
          info = rc.lines[i];
        }
        else
        {
          info = "Unified Standard Format Marker";
        }
        break;
      }
      case 4:
      {
        if (i < rc.lines.size ()) {
          printable_reuse = convert_to_bool (rc.lines[i]);
        }
        else
        {
          printable_reuse = true;
        }
        break;
      }
      case 5:
      {
        if (i < rc.lines.size ()) {
          fontsize = convert_to_double (rc.lines[i]);
        }
        else
        {
          fontsize = 12;
        }
        break;
      }
      case 6:
      {
        if (i < rc.lines.size ()) {
          italics = convert_to_bool (rc.lines[i]);
        }
        else
        {
          italics = false;
        }
        break;
      }
      case 7:
      {
        if (i < rc.lines.size ()) {
          bold = convert_to_bool (rc.lines[i]);
        }
        else
        {
          bold = false;
        }
        break;
      }
      case 8:
      {
        if (i < rc.lines.size ()) {
          superscript = convert_to_bool (rc.lines[i]);
        }
        else
        {
          superscript = false;
        }
        break;
      }
      case 9:
      {
        if (i < rc.lines.size ()) {
          startsparagraph_reuse = convert_to_bool (rc.lines[i]);
        }
        else
        {
          startsparagraph_reuse = false;
        }
        break;
      }
      case 10:
      {
        if (i < rc.lines.size ()) {
          justification = rc.lines[i];
        }
        else
        {
          justification = "left";
        }
        break;
      }
      case 11:
      {
        if (i < rc.lines.size ()) {
          spacebefore = convert_to_double (rc.lines[i]);
        }
        else
        {
          spacebefore = 0;
        }
        break;
      }
      case 12:
      {
        if (i < rc.lines.size ()) {
          spaceafter = convert_to_double (rc.lines[i]);
        }
        else
        {
          spaceafter = 0;
        }
        break;
      }
      case 13:
      {
        if (i < rc.lines.size ()) {
          leftmargin = convert_to_double (rc.lines[i]);
        }
        else
        {
          leftmargin = 0;
        }
        break;
      }
      case 14:
      {
        if (i < rc.lines.size ()) {
          rightmargin = convert_to_double (rc.lines[i]);
        }
        else
        {
          rightmargin = 0;
        }
        break;
      }
      case 15:
      {
        if (i < rc.lines.size ()) {
          firstlineindent = convert_to_double (rc.lines[i]);
        }
        else
        {
          firstlineindent = 0;
        }
        break;
      }
      case 16:
      {
        if (i < rc.lines.size ()) {
          spancolumns = convert_to_bool (rc.lines[i]);
        }
        else
        {
          spancolumns = false;
        }
        break;
      }
      case 17:
      {
        if (i < rc.lines.size ()) {
          type = StyleType (convert_to_int (rc.lines[i]));
        }
        else
        {
          type = stInlineText;
        }
        break;
      }
      
    }
  }
}


Style::~Style ()
{
  if (mywrite) {
    // If the object goes out of scope, it saves the settings.
    vector < string > output;
    output.push_back (VERSION);
    output.push_back ("There is no need for editing this file, because it is maintained by bibledit");
    output.push_back (name);
    output.push_back (info);
    output.push_back (convert_to_string (printable_reuse));
    output.push_back (convert_to_string (fontsize));
    output.push_back (convert_to_string (italics));
    output.push_back (convert_to_string (bold));
    output.push_back (convert_to_string (superscript));
    output.push_back (convert_to_string (startsparagraph_reuse));
    output.push_back (justification);
    output.push_back (convert_to_string (spacebefore));
    output.push_back (convert_to_string (spaceafter));
    output.push_back (convert_to_string (leftmargin));
    output.push_back (convert_to_string (rightmargin));
    output.push_back (convert_to_string (firstlineindent));
    output.push_back (convert_to_string (spancolumns));
    output.push_back (convert_to_string (int (type)));
    try
    {
      write_lines (filename, output);
    }
    catch (exception & ex)
    {
      cerr << ex.what () << endl;
    }
  }
}
