/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include <libgen.h>
#include <glib.h>
#include <config.h>
#include "constants.h"
#include "ustring.h"
#include "xmlfo-utils.h"
#include "date_time_utils.h"


void insert_empty_lines (vector<ustring> * lines)
{
  lines->push_back ("");
  lines->push_back ("");
}


XmlFoRoot::XmlFoRoot (vector<ustring> * lines, const string& font)
{
  mylines = lines;
  mylines->push_back ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
  ustring line;
  line = "<fo:root xmlns:fo=\"http://www.w3.org/1999/XSL/Format\" font-family=\"" + font + "\" font-size=\"12pt\" text-align=\"justify\">";
  mylines->push_back (line);  
  insert_empty_lines (mylines);
}


XmlFoRoot::~XmlFoRoot ()
{
  insert_empty_lines (mylines);
  mylines->push_back ("</fo:root>");
}


XmlFoLayoutMasterSet::XmlFoLayoutMasterSet (vector<ustring> * lines, Configuration * configuration, bool twocolumns)
{
  insert_empty_lines (lines);
  ustring line;
  lines->push_back ("  <fo:layout-master-set>");
  // Fill in paper size, margins.
  line = "    <fo:simple-page-master master-name=\"odd-page\" page-width=\""
       + convert_to_string (configuration->paper_width) + "cm\" page-height=\""
       + convert_to_string (configuration->paper_height) + "cm\">";
  lines->push_back (line);
  line = "      <fo:region-body margin-top=\""
       + convert_to_string (configuration->paper_top_margin) + "cm\" margin-bottom=\""
       + convert_to_string (configuration->paper_bottom_margin) + "cm\" margin-left=\""
       + convert_to_string (configuration->paper_left_margin) + "cm\" margin-right=\"" 
       + convert_to_string (configuration->paper_right_margin) + "cm\"";
  if (twocolumns)
    line.append (" column-count=\"2\"");
  line.append(">");
  lines->push_back (line);
  lines->push_back ("      </fo:region-body>");
  line = "      <fo:region-before region-name=\"header-odd\" extent=\"2cm\"/>";
  replace_text (line, "2", convert_to_string (configuration->paper_top_margin));
  lines->push_back (line);
  lines->push_back ("    </fo:simple-page-master>");
  line = "    <fo:simple-page-master master-name=\"even-page\" page-width=\""
       + convert_to_string (configuration->paper_width) + "cm\" page-height=\""
       + convert_to_string (configuration->paper_height) + "cm\">";
  lines->push_back (line);
  // Mirrored paper : swap left/right margins on even pages.
  line = "      <fo:region-body margin-top=\""
       + convert_to_string (configuration->paper_top_margin) + "cm\" margin-bottom=\""
       + convert_to_string (configuration->paper_bottom_margin) + "cm\" margin-left=\""
       + convert_to_string (configuration->paper_right_margin) + "cm\" margin-right=\""
       + convert_to_string (configuration->paper_left_margin) + "cm\"";
  if (twocolumns)
    line.append (" column-count=\"2\"");
  line.append(">");
  lines->push_back (line);
  lines->push_back ("      </fo:region-body>");
  line = "      <fo:region-before region-name=\"header-even\" extent=\"2cm\"/>";
  replace_text (line, "2", convert_to_string (configuration->paper_top_margin));
  lines->push_back (line);
  lines->push_back ("    </fo:simple-page-master>");
  lines->push_back ("    <fo:page-sequence-master master-name=\"text\">");
  lines->push_back ("      <fo:repeatable-page-master-alternatives>");
  lines->push_back ("        <fo:conditional-page-master-reference odd-or-even=\"odd\" master-reference=\"odd-page\"/>");
  lines->push_back ("        <fo:conditional-page-master-reference odd-or-even=\"even\" master-reference=\"even-page\"/>");
  lines->push_back ("      </fo:repeatable-page-master-alternatives>");
  lines->push_back ("    </fo:page-sequence-master>");
  lines->push_back ("  </fo:layout-master-set>");
  insert_empty_lines (lines);
}


XmlFoLayoutMasterSet::~XmlFoLayoutMasterSet ()
{
}


XmlFoPageSequence::XmlFoPageSequence (vector<ustring> * lines)
{
  mylines = lines;
  insert_empty_lines (mylines);
  mylines->push_back ("  <fo:page-sequence master-reference=\"text\" force-page-count=\"end-on-even\">");
  insert_empty_lines (mylines);
}


XmlFoPageSequence::~XmlFoPageSequence ()
{
  insert_empty_lines (mylines);
  mylines->push_back ("  </fo:page-sequence>");
  insert_empty_lines (mylines);
}


XmlFoStaticContent::XmlFoStaticContent (vector<ustring> * lines, Configuration * configuration)
{
  insert_empty_lines (lines);
  ustring date;
  date = date_time_julian_human_readable (date_time_julian_day_get_current (), false);
  ustring line;
  lines->push_back ("    <fo:static-content flow-name=\"header-odd\">");
  lines->push_back ("      <fo:table table-layout=\"fixed\" width=\"100%\">");
  line = "        <fo:table-column column-width=\"2cm\"/>";
  replace_text (line, "2", convert_to_string (configuration->paper_left_margin));
  lines->push_back (line);
  lines->push_back ("        <fo:table-column column-width=\"proportional-column-width(1)\"/>");
  lines->push_back ("        <fo:table-column column-width=\"proportional-column-width(1)\"/>");
  lines->push_back ("        <fo:table-column column-width=\"proportional-column-width(1)\"/>");
  line = "        <fo:table-column column-width=\"1cm\"/>";
  replace_text (line, "1", convert_to_string (configuration->paper_right_margin));
  lines->push_back (line);
  lines->push_back ("        <fo:table-body>");
  line = "          <fo:table-row height=\"2cm\">";
  replace_text (line, "2", convert_to_string (configuration->paper_top_margin));  
  lines->push_back (line);
  lines->push_back ("            <fo:table-cell/>");
  lines->push_back ("            <fo:table-cell/>");
  lines->push_back ("            <fo:table-cell display-align=\"center\">");
  lines->push_back ("              <fo:block text-align=\"center\">");
  lines->push_back ("                <fo:retrieve-marker retrieve-class-name=\"book\"/>");
  lines->push_back ("                <fo:retrieve-marker retrieve-class-name=\"chapter\"/>");
  lines->push_back ("              </fo:block>");
  lines->push_back ("            </fo:table-cell>");
  lines->push_back ("            <fo:table-cell display-align=\"center\">");
  lines->push_back ("              <fo:block text-align-last=\"end\">");
  if (configuration->printdate) {
    lines->push_back (date + " - - - ");
  }
  lines->push_back ("                <fo:page-number/>");
  lines->push_back ("              </fo:block>");
  lines->push_back ("            </fo:table-cell>");
  lines->push_back ("            <fo:table-cell/>");
  lines->push_back ("          </fo:table-row>");
  lines->push_back ("        </fo:table-body>");
  lines->push_back ("      </fo:table>");
  lines->push_back ("    </fo:static-content>");
  lines->push_back ("    <fo:static-content flow-name=\"header-even\">");
  lines->push_back ("      <fo:table table-layout=\"fixed\" width=\"100%\">");
  line = "        <fo:table-column column-width=\"1cm\"/>";
  replace_text (line, "1", convert_to_string (configuration->paper_right_margin));
  lines->push_back (line);
  lines->push_back ("        <fo:table-column column-width=\"proportional-column-width(1)\"/>");
  lines->push_back ("        <fo:table-column column-width=\"proportional-column-width(1)\"/>");
  lines->push_back ("        <fo:table-column column-width=\"proportional-column-width(1)\"/>");
  line = "        <fo:table-column column-width=\"2cm\"/>";
  replace_text (line, "2", convert_to_string (configuration->paper_left_margin));
  lines->push_back (line);
  lines->push_back ("        <fo:table-body>");
  line = "          <fo:table-row height=\"2cm\">";
  replace_text (line, "2", convert_to_string (configuration->paper_top_margin));  
  lines->push_back (line);
  lines->push_back ("            <fo:table-cell/>");
  lines->push_back ("            <fo:table-cell display-align=\"center\">");
  lines->push_back ("              <fo:block text-align=\"start\">");
  lines->push_back ("                <fo:page-number/>");
  if (configuration->printdate) {
    lines->push_back (" - - - " + date);
  }
  lines->push_back ("              </fo:block>");
  lines->push_back ("            </fo:table-cell>");
  lines->push_back ("            <fo:table-cell display-align=\"center\">");
  lines->push_back ("              <fo:block text-align=\"center\">");
  lines->push_back ("                <fo:retrieve-marker retrieve-class-name=\"book\"/>");
  lines->push_back ("                <fo:retrieve-marker retrieve-class-name=\"chapter\"/>");
  lines->push_back ("              </fo:block>");
  lines->push_back ("            </fo:table-cell>");
  lines->push_back ("            <fo:table-cell/>");
  lines->push_back ("            <fo:table-cell/>");
  lines->push_back ("          </fo:table-row>");
  lines->push_back ("        </fo:table-body>");
  lines->push_back ("      </fo:table>");
  lines->push_back ("    </fo:static-content>");
  insert_empty_lines (lines);
}


XmlFoStaticContent::~XmlFoStaticContent ()
{
}


XmlFoFlow::XmlFoFlow (vector<ustring> * lines)
{
  mylines = lines;
  insert_empty_lines (mylines);
  mylines->push_back ("    <fo:flow flow-name=\"xsl-region-body\">");
  insert_empty_lines (mylines);
}


XmlFoFlow::~XmlFoFlow ()
{
  insert_empty_lines (mylines);
  mylines->push_back ("    </fo:flow>");
  insert_empty_lines (mylines);
}


XmlFoBlock::XmlFoBlock (vector<ustring> * lines, double fontsize,
                        bool italics, bool bold, string justification,
                        double spacebefore, double spaceafter,
                        double leftmargin, double rightmargin,
                        double firstlineindent, bool spancolumns)
{
  mylines = lines;
  insert_empty_lines (mylines);
  ustring line;
  line = "      <fo:block";
  // Fontsize
  line.append (" font-size=\"");
  line.append (convert_to_string (fontsize));
  line.append ("pt\"");
  // Italics
  if (italics) {
    line.append (" font-style=\"italic\"");
  }
  // Bold
  if (bold) {
    line.append (" font-weight=\"bold\"");
  }
  // Justification
  if (justification == "last-justify")
    line.append (" text-align-last=\"justify\"");
  else {
    if (!justification.empty()) {
      line.append (" text-align=\"");
      line.append (justification);
      line.append ("\"");
    }
  }
  // Space before
  if (spacebefore != 0) {
    line.append (" space-before=\"");
    line.append (convert_to_string (spacebefore));
    line.append ("mm\"");
  }
  // Space after
  if (spaceafter != 0) {
    line.append (" space-after=\"");
    line.append (convert_to_string (spaceafter));
    line.append ("mm\"");
  }
  // Left margin
  if (leftmargin != 0) {
    line.append (" start-indent=\"");
    line.append (convert_to_string (leftmargin));
    line.append ("mm\"");
  }
  // Right margin
  if (rightmargin != 0) {
    line.append (" end-indent=\"");
    line.append (convert_to_string (rightmargin));
    line.append ("mm\"");
  }
  // First line indent
  if (firstlineindent != 0) {
    line.append (" text-indent=\"");
    line.append (convert_to_string (firstlineindent));
    line.append ("mm\"");
  }
  // Spanning of columns.
  if (spancolumns)
    line.append (" span=\"all\"");
  line.append (">");
  mylines->push_back (line);
}


XmlFoBlock::~XmlFoBlock ()
{
  mylines->push_back ("      </fo:block>");
  insert_empty_lines (mylines);
}


ustring XmlFoInlineText (const ustring& line, double fontsize, bool italics, bool bold, bool superscript)
{
  ustring result;
  result = "<fo:inline font-size=\"FontSizept\"";
  replace_text (result, "FontSize", convert_to_string (fontsize));
  if (italics) {
    result.append (" font-style=\"italic\"");
  }
  if (bold) {
    result.append (" font-weight=\"bold\"");
  }
  if (superscript) {
    result.append (" vertical-align=\"super\"");
  }
  result.append (">");
  result.append (line);
  result.append ("</fo:inline>");
  return result;
}
